/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.components;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Stack;
import org.apache.avalon.excalibur.component.ExcaliburComponentManager;
import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.component.Recomposable;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.environment.Environment;
/* The new source resolving (will go online after 2.0.2)
import java.net.MalformedURLException;
import org.apache.avalon.excalibur.source.Source;
import org.apache.avalon.excalibur.source.SourceResolver;
*/

import org.xml.sax.SAXException;

/**
 * Cocoon Component Manager.
 * This manager extends the <code>ExcaliburComponentManager</code>
 * by a special lifecycle handling for a RequestLifecycleComponent
 * and by handling the lookup of the <code>SourceResolver</code> (in development)
 *
 * @author <a href="mailto:cziegeler@org.com">Carsten Ziegeler</a>
 * @version CVS $Id: CocoonComponentManager.java,v 1.11 2002/03/15 13:39:51 cziegeler Exp $
 */
public final class CocoonComponentManager
extends ExcaliburComponentManager
/* The new source resolving (will go online after 2.0.2)
implements SourceResolver
*/
{
    /** The environment information */
    private static InheritableThreadLocal environmentStack = new InheritableThreadLocal();

    /** The configured <code>SourceResolver</code> */
    /* The new source resolving (will go online after 2.0.2)
    private SourceResolver sourceResolver;
    */

    /** Create the ComponentManager */
    public CocoonComponentManager()
    {
        super( null, Thread.currentThread().getContextClassLoader() );
    }

    /** Create the ComponentManager with a Classloader */
    public CocoonComponentManager( final ClassLoader loader )
    {
        super( null, loader );
    }

    /** Create the ComponentManager with a Classloader and parent ComponentManager */
    public CocoonComponentManager( final ComponentManager manager, final ClassLoader loader )
    {
        super(manager, loader);
    }

    /** Create the ComponentManager with a parent ComponentManager */
    public CocoonComponentManager(final ComponentManager manager)
    {
        super(manager);
    }

    private static final Integer ONE = new Integer(1);
    private static final Integer TWO = new Integer(2);

    /**
     * This hook must be called by the sitemap each time a sitemap is entered
     */
    public static void enterEnvironment(Environment env, Map objectModel) {
        if (environmentStack.get() == null) {
            environmentStack.set(new Stack());
        }
        final Stack stack = (Stack)environmentStack.get();
        if ( !stack.empty() ) {
            final Object[] objects = (Object[])stack.peek();
            if ( objects[1] == objectModel ) {
                stack.push(new Object[] {env, objectModel, objects[2], TWO});
                return;
            }
        }
        stack.push(new Object[] {env, objectModel, new HashMap(5), ONE});
    }

    /**
     * This hook must be called by the sitemap each time a sitemap is left
     */
    public static void leaveEnvironment() {
        final Stack stack = (Stack)environmentStack.get();
        if ( null != stack && !stack.empty()) {
            final Object[] objects = (Object[])stack.pop();
            if (objects[3] == ONE) {
                final Map components = (Map)objects[2];
                final Iterator iter = components.values().iterator();
                while (iter.hasNext()) {
                    final Object[] o = (Object[])iter.next();
                    final Component component = (Component)o[0];
                    ((CocoonComponentManager)o[1]).releaseRLComponent( component );
                }
            }
        }
    }

    /**
     * Return an instance of a component based on a Role.  The Role is usually the Interface's
     * Fully Qualified Name(FQN)--unless there are multiple Components for the same Role.  In that
     * case, the Role's FQN is appended with "Selector", and we return a ComponentSelector.
     */
    public Component lookup( final String role )
    throws ComponentException {
        /* The new source resolving (will go online after 2.0.2)
        if ( role.equals(SourceResolver.ROLE) ) {
            if ( null == this.sourceResolver ) {
                this.sourceResolver = (SourceResolver) super.lookup( role );
            }
            return this;
        }
        */
        final Stack stack = (Stack)environmentStack.get();
        if ( null != stack && !stack.empty()) {
            final Object[] objects = (Object[])stack.peek();
            final Map components = (Map)objects[2];
            final Object[] o = (Object[])components.get(role);
            if ( null != o  ) {
                final Component component = (Component) o[0];
                if (null != component) {
                    return component;
                }
            }
        }

        final Component component = super.lookup( role );
        if (null != component && component instanceof RequestLifecycleComponent) {
            if (stack == null || stack.empty()) {
                throw new ComponentException("ComponentManager has no Environment Stack.");
            }
            final Object[] objects = (Object[]) stack.peek();
            final Map components = (Map)objects[2];
            try {
                if (component instanceof Recomposable) {
                    ((Recomposable) component).recompose(this);
                }
                ((RequestLifecycleComponent) component).setup((org.apache.cocoon.environment.SourceResolver)objects[0],
                                                              (Map)objects[1]);
            } catch (Exception local) {
                throw new ComponentException("Exception during setup of RequestLifecycleComponent with role '"+role+"'", local);
            }
            components.put(role, new Object[] {component, this});
        }
        return component;
    }

    /**
     * Release a Component.  This implementation makes sure it has a handle on the propper
     * ComponentHandler, and let's the ComponentHandler take care of the actual work.
     */
    public void release( final Component component ) {
        if( null == component ) {
            return;
        }

        if ( component instanceof RequestLifecycleComponent) {
            return;
        }
        /* The new source resolving (will go online after 2.0.2)
        if ( component == this ) {
            return;
        }
        */
        super.release( component);
    }

    /**
     * Dispose
     */
    /* The new source resolving (will go online after 2.0.2)
    public void dispose() {
        if ( null != this.sourceResolver ) {
            super.release( this.sourceResolver );
            this.sourceResolver = null;
        }
        super.dispose();
    }
    */

    /**
     * Release a RequestLifecycleComponent
     */
    protected void releaseRLComponent( final Component component ) {
        super.release( component );
    }

    /**
     * Get a <code>Source</code> object.
     */
    /* The new source resolving (will go online after 2.0.2)
    public Source resolve(final String location)
    throws MalformedURLException, IOException, ComponentException {
        return this.resolve(location, null, null);
    }
    */

    /**
     * Get a <code>Source</code> object.
     */
    /* The new source resolving (will go online after 2.0.2)
    public Source resolve(final String location,
                          String baseURI,
                          final Map    parameters)
    throws MalformedURLException, IOException, ComponentException {
        if (baseURI == null) {
            final Stack stack = (Stack)environmentStack.get();
            if ( null != stack && !stack.empty()) {
                final Object[] objects = (Object[])stack.peek();
                baseURI = ((Environment)objects[0]).getContext().toExternalForm();
            }
        }
        return this.sourceResolver.resolve(location, baseURI, parameters);
    }
    */

    /**
     * Releases a resolved resource
     */
    /* The new source resolving (will go online after 2.0.2)
    public void release( final Source source ) {
        this.sourceResolver.release( source );
    }
    */
}
