/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.components.pipeline;

import org.apache.avalon.excalibur.pool.Recyclable;
import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.component.Recomposable;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.environment.Environment;

/** A <code>StreamPipeline</code> either
 * <ul>
 *  <li>collects a <code>Reader</code> and let it produce a character stream</li>
 *  <li>or connects an <code>EventPipeline</code> with a
 *  <code>Serializer</code> and let them produce the byte stream</li>
 * </UL>
 * <p>
 * A <code>StreamPipeline</code> is <code>Recomposable</code> since the
 * <code>ComponentManager</code> used to get the reader or serializer
 * depends on the pipeline assembly engine where they are defined (i.e.
 * a given sitemap file).
 *
 * @author <a href="mailto:Giacomo.Pati@pwr.ch">Giacomo Pati</a>
 * @version CVS $Id: StreamPipeline.java,v 1.4 2002/02/22 07:00:11 cziegeler Exp $
 */
public interface StreamPipeline extends Component, Recomposable, Recyclable {

    String ROLE = "org.apache.cocoon.components.pipeline.StreamPipeline";

    /**
     * Process the given <code>Environment</code> producing the output
     */
    boolean process(Environment environment)
    throws Exception;

    /**
     * Set the <code>EventPipeline</code>
     */
    void setEventPipeline (EventPipeline eventPipeline) throws Exception;

    /**
     * Get the <code>EventPipeline</code>
     */
    EventPipeline getEventPipeline ();

    /**
     * Set the reader for this pipeline
     */
    void setReader (String role, String source, Parameters param) throws Exception;

    /**
     * Set the reader for this pipeline
     */
    void setReader (String role, String source, Parameters param, String mimeType) throws Exception;

    /**
     * Set the serializer for this pipeline
     */
    void setSerializer (String role, String source, Parameters param) throws Exception;

    /**
     * Set the serializer for this pipeline
     */
    void setSerializer (String role, String source, Parameters param, String mimeType) throws Exception;
}
