/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.components.profiler;

import java.util.Arrays;

/**
 * Represents data collected about one pipeline.
 *
 * @author <a href="mailto:vgritsenko@apache.org">Vadim Gritsenko</a>
 * @version CVS $Id: ProfilerResult.java,v 1.4 2002/02/22 07:00:11 cziegeler Exp $
 */
public class ProfilerResult
{
    private String uri;
    private String[] roles;
    private String[] sources;

    private int count;
    private long[] totalTime;
    private long[][] latestTimes;
    private int latestCurrent;

    public ProfilerResult(String uri, int latestResultsCount)
    {
        this.uri = uri;
        if(latestResultsCount > 0)
            this.latestTimes = new long[latestResultsCount][];
    }

    public void addData(ProfilerData data)
    {
        ProfilerData.Entry[] entries = data.getEntries();
        synchronized(this){
            if(totalTime == null || totalTime.length != entries.length){
                // Reinitialize arrays
                totalTime = new long[entries.length];
                roles = new String[entries.length];
                sources = new String[entries.length];
                for(int i=0; i<entries.length; i++){
                    roles[i] = entries[i].role;
                    if(roles[i] == null)
                        roles[i] = entries[i].component.getClass().getName();
                    sources[i] = entries[i].source;
                }

                // Clear latest times
                if(latestTimes != null)
                    Arrays.fill(latestTimes, 0, latestTimes.length, null);
                latestCurrent = 0;

                // Clear counter
                count = 0;
            }

            // Adjust time. It's cumulative towards 0 index in data object
            for(int i=0; i<entries.length-1; i++)
                entries[i].time -= entries[i+1].time;

            long[] latest = new long[entries.length];
            for(int i=0; i<entries.length; i++)
                totalTime[i] += latest[i] = entries[i].time;

            if(latestTimes != null){
                latestTimes[latestCurrent++] = latest;
                if(latestCurrent >= latestTimes.length) latestCurrent = 0;
            }

            count++;
        }
    }

    public String getURI()
    {
        return uri;
    }

    public String[] getRoles()
    {
        return roles;
    }

    public String[] getSources()
    {
        return sources;
    }

    public int getCount()
    {
        return count;
    }

    public long[] getTotalTime()
    {
        return totalTime;
    }

    public long[][] getLastTimes()
    {
        return latestTimes;
    }
/*
    public void report()
    {
        System.err.println("-------------------------------------------------------------------------------");
        System.err.println("PROFILER TOTALS FOR: " + uri);

        if(totalTime != null){
            long time = 0;
            for(int i=0; i<totalTime.length; i++){
                System.err.println("PROFILER TOTALS: " + totalTime[i] + "\tFOR " + roles[i] + "\t" + sources[i]);
                time += totalTime[i];
            }
            System.err.println("PROFILER TOTALS: " + time + " TIMES");
            System.err.println("PROFILER TOTALS: " + count + " REQUESTS");
        }
    }
*/
}
