/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.components.profiler;

import org.apache.avalon.excalibur.pool.Recyclable;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.components.saxconnector.SAXConnector;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.cocoon.xml.AbstractXMLPipe;
import org.apache.cocoon.xml.XMLConsumer;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.io.IOException;
import java.util.Map;

/**
 * This SAX connector measures time taken by the following SAX handler.
 * @author <a href="vgritsenko@apache.org">Vadim Gritsenko</a>
 * @version CVS $Id: ProfilingSAXConnector.java,v 1.4 2002/02/22 07:00:11 cziegeler Exp $
 */
public class ProfilingSAXConnector extends AbstractXMLPipe
    implements Recyclable, SAXConnector
{
    private ProfilerData data;
    private int index;
    private long time;
    private long total;

    /**
     * Setup this SAXConnector.
     */
    public void setup(SourceResolver resolver, Map objectModel, String src, Parameters params)
        throws ProcessingException, SAXException, IOException
    {
        this.data = (ProfilerData)objectModel.get("profiler");
    }

    /**
     * Recycle the SAXConnector
     */
    public void recycle()
    {
        this.data = null;
        this.time = this.total = 0;
        this.index = -1;
        super.recycle();
    }

    public void setConsumer(XMLConsumer consumer)
    {
        super.setConsumer(consumer);
        if(this.data != null)
            this.index = this.data.indexOf(consumer);
        else
            this.index = -1;
    }


    public void startDocument() throws SAXException
    {
        time = System.currentTimeMillis();
        super.startDocument();
        total += System.currentTimeMillis() - time;
    }

    public void endDocument() throws SAXException
    {
        time = System.currentTimeMillis();
        super.endDocument();
        total += System.currentTimeMillis() - time;
        if(index != -1)
            data.setTime(index, total);
    }

    public void startPrefixMapping(String prefix, String uri) throws SAXException
    {
        time = System.currentTimeMillis();
        super.startPrefixMapping(prefix, uri);
        total += System.currentTimeMillis() - time;
    }

    public void endPrefixMapping(String prefix) throws SAXException
    {
        time = System.currentTimeMillis();
        super.endPrefixMapping(prefix);
        total += System.currentTimeMillis() - time;
    }

    public void startElement(String uri, String loc, String raw, Attributes a) throws SAXException
    {
        time = System.currentTimeMillis();
        super.startElement(uri, loc, raw, a);
        total += System.currentTimeMillis() - time;
    }

    public void endElement(String uri, String loc, String raw) throws SAXException
    {
        time = System.currentTimeMillis();
        super.endElement(uri, loc, raw);
        total += System.currentTimeMillis() - time;
    }

    public void characters(char c[], int start, int len) throws SAXException
    {
        time = System.currentTimeMillis();
        super.characters(c, start, len);
        total += System.currentTimeMillis() - time;
    }

    public void ignorableWhitespace(char c[], int start, int len) throws SAXException
    {
        time = System.currentTimeMillis();
        super.ignorableWhitespace(c, start, len);
        total += System.currentTimeMillis() - time;
    }

    public void processingInstruction(String target, String data) throws SAXException
    {
        time = System.currentTimeMillis();
        super.processingInstruction(target, data);
        total += System.currentTimeMillis() - time;
    }

    public void skippedEntity(String name) throws SAXException
    {
        time = System.currentTimeMillis();
        super.skippedEntity(name);
        total += System.currentTimeMillis() - time;
    }

    public void startDTD(String name, String publicId, String systemId) throws SAXException
    {
        time = System.currentTimeMillis();
        super.startDTD(name, publicId, systemId);
        total += System.currentTimeMillis() - time;
    }

    public void endDTD() throws SAXException
    {
        time = System.currentTimeMillis();
        super.endDTD();
        total += System.currentTimeMillis() - time;
    }

    public void startEntity(String name) throws SAXException
    {
        time = System.currentTimeMillis();
        super.startEntity(name);
        total += System.currentTimeMillis() - time;
    }

    public void endEntity(String name) throws SAXException
    {
        time = System.currentTimeMillis();
        super.endEntity(name);
        total += System.currentTimeMillis() - time;
    }

    public void startCDATA() throws SAXException
    {
        time = System.currentTimeMillis();
        super.startCDATA();
        total += System.currentTimeMillis() - time;
    }

    public void endCDATA() throws SAXException
    {
        time = System.currentTimeMillis();
        super.endCDATA();
        total += System.currentTimeMillis() - time;
    }

    public void comment(char ch[], int start, int len) throws SAXException
    {
        time = System.currentTimeMillis();
        super.comment(ch, start, len);
        total += System.currentTimeMillis() - time;
    }
}
