/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.components.request.multipart;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletRequest;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.security.Principal;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Map;
import java.util.Vector;

/**
 * Servlet request wrapper for multipart parser.
 *
 * @author Jeroen ter Voorde
 * @author <a href="mailto:j.tervoorde@home.nl">Jeroen ter Voorde</a>
 * @version CVS $Id: MultipartRequestWrapper.java,v 1.1 2002/02/27 20:21:22 dims Exp $
 */
public class MultipartRequestWrapper
        implements ServletRequest, HttpServletRequest {

    /** Field CONTENTHEADER           */
    private final String CONTENTHEADER = "multipart/form-data";

    /** Field request           */
    private HttpServletRequest request = null;

    /** Field values           */
    private MultipartParser values = null;

    /**
     * Constructor MultipartRequestWrapper
     *
     * @param request
     * @param saveUploadedFilesToDisk
     * @param uploadDirectory
     * @param allowOverwrite
     * @param silentlyRename
     * @param maxUploadSize
     *
     * @throws IOException
     * @throws MultipartException
     */
    public MultipartRequestWrapper(
            HttpServletRequest request, boolean saveUploadedFilesToDisk, File uploadDirectory, boolean allowOverwrite, boolean silentlyRename, int maxUploadSize)
            throws IOException, MultipartException {

        this.request = request;

        String contentType = request.getContentType();

        if ((contentType != null)
                && (contentType.toLowerCase().indexOf(CONTENTHEADER) > -1)) {
            values = new MultipartParser(request, saveUploadedFilesToDisk,
                    uploadDirectory, allowOverwrite,
                    silentlyRename, maxUploadSize);
        }
    }

    /**
     * Method get
     *
     * @param name
     *
     * @return
     */
    public Object get(String name) {

        Object result = null;

        if (values != null) {
            result = values.get(name);

            if (result instanceof Vector) {
                if (((Vector) result).size() == 1) {
                    return ((Vector) result).elementAt(0);
                } else {
                    return result;
                }
            }
        } else {
            String[] array = request.getParameterValues(name);
            Vector vec = new Vector();

            if (array != null) {
                for (int i = 0; i < array.length; i++) {
                    vec.addElement(array[i]);
                }

                if (vec.size() == 1) {
                    result = vec.elementAt(0);
                } else {
                    result = vec;
                }
            }
        }

        return result;
    }

    /**
     * Method getParameterNames
     *
     * @return
     */
    public Enumeration getParameterNames() {

        if (values != null) {
            return values.keys();
        } else {
            return request.getParameterNames();
        }
    }

    /**
     * Method getParameter
     *
     * @param name
     *
     * @return
     */
    public String getParameter(String name) {

        Object value = get(name);
        String result = null;

        if (value != null) {
            if (value instanceof Vector) {
                value = ((Vector) value).elementAt(0);
            }

            result = value.toString();
        }

        return result;
    }

    /**
     * Method getParameterValues
     *
     * @param name
     *
     * @return
     */
    public String[] getParameterValues(String name) {

        if (values != null) {
            Object value = get(name);

            if (value != null) {
                if (value instanceof Vector) {
                    return (String[]) ((Vector) value).toArray();
                } else {
                    return new String[]{value.toString()};
                }
            }

            return null;
        } else {
            return request.getParameterValues(name);
        }
    }

    /**
     * Method getRequest
     *
     * @return
     */
    public HttpServletRequest getRequest() {
        return request;
    }

    /**
     * Method getAttribute
     *
     * @param name
     *
     * @return
     */
    public Object getAttribute(String name) {
        return request.getAttribute(name);
    }

    /**
     * Method getAttributeNames
     *
     * @return
     */
    public Enumeration getAttributeNames() {
        return request.getAttributeNames();
    }

    /**
     * Method getCharacterEncoding
     *
     * @return
     */
    public String getCharacterEncoding() {
        return request.getCharacterEncoding();
    }

    /**
     * Method getContentLength
     *
     * @return
     */
    public int getContentLength() {
        return request.getContentLength();
    }

    /**
     * Method getContentType
     *
     * @return
     */
    public String getContentType() {
        return request.getContentType();
    }

    /**
     * Method getInputStream
     *
     * @return
     *
     * @throws IOException
     */
    public ServletInputStream getInputStream() throws IOException {
        return request.getInputStream();
    }

    /**
     * Method getProtocol
     *
     * @return
     */
    public String getProtocol() {
        return request.getProtocol();
    }

    /**
     * Method getScheme
     *
     * @return
     */
    public String getScheme() {
        return request.getScheme();
    }

    /**
     * Method getServerName
     *
     * @return
     */
    public String getServerName() {
        return request.getServerName();
    }

    /**
     * Method getServerPort
     *
     * @return
     */
    public int getServerPort() {
        return request.getServerPort();
    }

    /**
     * Method getReader
     *
     * @return
     *
     * @throws IOException
     */
    public BufferedReader getReader() throws IOException {
        return request.getReader();
    }

    /**
     * Method getRemoteAddr
     *
     * @return
     */
    public String getRemoteAddr() {
        return request.getRemoteAddr();
    }

    /**
     * Method getRemoteHost
     *
     * @return
     */
    public String getRemoteHost() {
        return request.getRemoteHost();
    }

    /**
     * Method setAttribute
     *
     * @param name
     * @param o
     */
    public void setAttribute(String name, Object o) {
        request.setAttribute(name, o);
    }

    /**
     * Method removeAttribute
     *
     * @param name
     */
    public void removeAttribute(String name) {
        request.removeAttribute(name);
    }

    /**
     * Method getLocale
     *
     * @return
     */
    public Locale getLocale() {
        return request.getLocale();
    }

    /**
     * Method getLocales
     *
     * @return
     */
    public Enumeration getLocales() {
        return request.getLocales();
    }

    /**
     * Method isSecure
     *
     * @return
     */
    public boolean isSecure() {
        return request.isSecure();
    }

    /**
     * Method getRequestDispatcher
     *
     * @param path
     *
     * @return
     */
    public RequestDispatcher getRequestDispatcher(String path) {
        return request.getRequestDispatcher(path);
    }

    /**
     * Method getRealPath
     *
     * @param path
     *
     * @return
     */
    public String getRealPath(String path) {
        return request.getRealPath(path);
    }

    /**
     * Method getAuthType
     *
     * @return
     */
    public String getAuthType() {
        return request.getAuthType();
    }

    /**
     * Method getCookies
     *
     * @return
     */
    public Cookie[] getCookies() {
        return request.getCookies();
    }

    /**
     * Method getDateHeader
     *
     * @param name
     *
     * @return
     */
    public long getDateHeader(String name) {
        return request.getDateHeader(name);
    }

    /**
     * Method getHeader
     *
     * @param name
     *
     * @return
     */
    public String getHeader(String name) {
        return request.getHeader(name);
    }

    /**
     * Method getHeaders
     *
     * @param name
     *
     * @return
     */
    public Enumeration getHeaders(String name) {
        return request.getHeaders(name);
    }

    /**
     * Method getHeaderNames
     *
     * @return
     */
    public Enumeration getHeaderNames() {
        return request.getHeaderNames();
    }

    /**
     * Method getIntHeader
     *
     * @param name
     *
     * @return
     */
    public int getIntHeader(String name) {
        return request.getIntHeader(name);
    }

    /**
     * Method getMethod
     *
     * @return
     */
    public String getMethod() {
        return request.getMethod();
    }

    /**
     * Method getPathInfo
     *
     * @return
     */
    public String getPathInfo() {
        return request.getPathInfo();
    }

    /**
     * Method getPathTranslated
     *
     * @return
     */
    public String getPathTranslated() {
        return request.getPathTranslated();
    }

    /**
     * Method getContextPath
     *
     * @return
     */
    public String getContextPath() {
        return request.getContextPath();
    }

    /**
     * Method getQueryString
     *
     * @return
     */
    public String getQueryString() {
        return request.getQueryString();
    }

    /**
     * Method getRemoteUser
     *
     * @return
     */
    public String getRemoteUser() {
        return request.getRemoteUser();
    }

    /**
     * Method isUserInRole
     *
     * @param role
     *
     * @return
     */
    public boolean isUserInRole(String role) {
        return request.isUserInRole(role);
    }

    /**
     * Method getUserPrincipal
     *
     * @return
     */
    public Principal getUserPrincipal() {
        return request.getUserPrincipal();
    }

    /**
     * Method getRequestedSessionId
     *
     * @return
     */
    public String getRequestedSessionId() {
        return request.getRequestedSessionId();
    }

    /**
     * Method getRequestURI
     *
     * @return
     */
    public String getRequestURI() {
        return request.getRequestURI();
    }

    /**
     * Method getServletPath
     *
     * @return
     */
    public String getServletPath() {
        return request.getServletPath();
    }

    /**
     * Method getSession
     *
     * @param create
     *
     * @return
     */
    public HttpSession getSession(boolean create) {
        return request.getSession(create);
    }

    /**
     * Method getSession
     *
     * @return
     */
    public HttpSession getSession() {
        return request.getSession();
    }

    /**
     * Method isRequestedSessionIdValid
     *
     * @return
     */
    public boolean isRequestedSessionIdValid() {
        return request.isRequestedSessionIdValid();
    }

    /**
     * Method isRequestedSessionIdFromCookie
     *
     * @return
     */
    public boolean isRequestedSessionIdFromCookie() {
        return request.isRequestedSessionIdFromCookie();
    }

    /**
     * Method isRequestedSessionIdFromURL
     *
     * @return
     */
    public boolean isRequestedSessionIdFromURL() {
        return request.isRequestedSessionIdFromURL();
    }

    /**
     * Method isRequestedSessionIdFromUrl
     *
     * @return
     */
    public boolean isRequestedSessionIdFromUrl() {
        return request.isRequestedSessionIdFromURL();
    }

    /**
     * Method getParameterMap
     *
     * @return
     */
    public Map getParameterMap() {
        // FIXME:
        return null;
    }

    /**
     * Method setCharacterEncoding
     *
     * @param s
     */
    public void setCharacterEncoding(String s) {
        // FIXME:
    }

    /**
     * Method getRequestURL
     *
     * @return
     */
    public StringBuffer getRequestURL() {
        // FIXME:
        return null;
    }
}
