/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.components.treeprocessor.sitemap;

import org.apache.avalon.framework.activity.Initializable;

import org.apache.cocoon.environment.Environment;

import org.apache.cocoon.sitemap.PatternException;

import org.apache.cocoon.components.treeprocessor.AbstractProcessingNode;
import org.apache.cocoon.components.treeprocessor.CategoryNode;
import org.apache.cocoon.components.treeprocessor.InvokeContext;
import org.apache.cocoon.components.treeprocessor.MapStackResolver;
import org.apache.cocoon.components.treeprocessor.ParameterizableProcessingNode;
import org.apache.cocoon.components.treeprocessor.ProcessingNode;

import java.util.List;
import java.util.Map;

/**
 *
 * @author <a href="mailto:sylvain@apache.org">Sylvain Wallez</a>
 * @version CVS $Id: CallNode.java,v 1.2 2002/03/17 21:55:22 sylvain Exp $
 */

public class CallNode extends AbstractProcessingNode
    implements Initializable, ParameterizableProcessingNode {

    /** The parameters of this node */
    private Map parameters;

    /** The 'resource' attribute */
    private String resourceName;

    private MapStackResolver resourceResolver;

    /** The category node */
    private CategoryNode resources;

    private ProcessingNode resourceNode;

    public void setParameters(Map parameterMap) {
        this.parameters = parameterMap;
    }

    public void setResource(CategoryNode resources, String resourceName) throws Exception {
        this.resourceName = resourceName;
        this.resources = resources;
    }

    public void initialize() throws Exception {
        if (MapStackResolver.needsResolve(this.resourceName)) {
            // Will always be resolved at invoke time
            this.resourceResolver = MapStackResolver.getResolver(this.resourceName);
        } else {
            // Static name : get it now
            this.resourceNode = this.resources.getNodeByName(MapStackResolver.unescape(this.resourceName));
        }
    }

    public final boolean invoke(Environment env, InvokeContext context)
      throws Exception {

        // Resolve parameters
        if (this.parameters != null) {
            // In redirect-to a level is added to the map stack *only* if the
            // 'target' attribute is present.
            Map params = MapStackResolver.resolveMap(this.parameters, context.getMapStack());
            context.pushMap(params);
        }

        boolean result;

        try {
            if (this.resourceNode != null) {
                // Static resource name
                result = this.resourceNode.invoke(env, context);
    
            } else {
                // Resolved resource name
                String name = this.resourceResolver.resolve(context.getMapStack());
    
                if (getLogger().isInfoEnabled()) {
                    getLogger().info("Calling resource " + name);
                }
    
                result = this.resources.invokeByName(name, env, context);
            }
        } finally {
            if (this.parameters != null) {
                context.popMap();
            }
        }
        return result;
    }
}
