/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.components.xscript;

import java.util.HashMap;

/**
 * <code>XScriptVariableScope</code> maintains variables in a given
 * scope. A variable has a unique name within a scope, but multiple
 * variables with the same name can exist within different scopes.
 *
 * @author <a href="mailto:ovidiu@cup.hp.com">Ovidiu Predescu</a>
 * @version CVS $Id: XScriptVariableScope.java,v 1.4 2002/02/22 07:00:15 cziegeler Exp $
 * @since August 4, 2001
 */
public class XScriptVariableScope
{
  /**
   * The variables store; each entry is <code>String</code>
   * representing the name of the variable, with the corresponding
   * value an {@link XScriptObject}.
   */
  HashMap variables = new HashMap();

  /**
   * Define a new variable or overwrite the value of an existing
   * variable in this scope.
   *
   * @param name a <code>String</code> value
   * @param value a <code>{@link XScriptObject}</code> value
   */
  public synchronized void put(String name, XScriptObject value)
  {
    variables.put(name, value);
  }

  /**
   * Obtains the value of the XScript <code>name</code> variable.
   *
   * @param name a <code>String</code> value
   * @return a <code>{@link XScriptObject}</code> value
   */
  public synchronized XScriptObject get(String name)
    throws IllegalArgumentException
  {
    if (!variables.containsKey(name))
      throw new IllegalArgumentException("Cannot find variable '"
                                         + name + "'!");
    return (XScriptObject)variables.get(name);
  }

  /**
   * Removes the XScript variable that's accessible via
   * <code>name</code>.
   *
   * @param name a <code>String</code> value
   */
  public synchronized void remove(String name)
  {
    variables.remove(name);
  }

  public synchronized boolean defines(String name)
  {
    return variables.containsKey(name);
  }
}
