package org.apache.cocoon.components.flow;

import org.apache.avalon.excalibur.component.ExcaliburComponentSelector;
import org.apache.avalon.excalibur.logger.LogKitManageable;
import org.apache.avalon.excalibur.logger.LogKitManager;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.component.Composable;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.Loggable;
import org.apache.avalon.framework.parameters.ParameterException;
import org.apache.avalon.framework.parameters.Parameterizable;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.log.Logger;

public class InterpreterSelector extends ExcaliburComponentSelector
  implements Configurable, ThreadSafe, Composable, LogKitManageable
{
  LogKitManager logkit;
  ComponentManager manager;
  String defaultLanguage;
  
  ClassLoader loader = Thread.currentThread().getContextClassLoader();

  public void setLogKitManager(LogKitManager logkit)
  {
    this.logkit = logkit;
  }

  public void configure(Configuration config)
    throws ConfigurationException
  {
    defaultLanguage = config.getAttribute("default", null);
    boolean reloadScripts = config.getAttributeAsBoolean("reload-scripts", false);
    long checkTime = config.getAttributeAsLong("check-time", 1000L);
    String loggerName = config.getAttribute("logger", null);
    Logger loggerUsed = getLogger();
    if (loggerName != null) {
      loggerUsed = logkit.getLogger(loggerName);
    }

    Configuration[] components = config.getChildren("component-instance");
    if (components.length == 0)
      throw new ConfigurationException("No languages defined!");

    for (int i = 0; i < components.length; i++) {
      Configuration component = components[i];
      String name = component.getAttribute("name");
      String className = component.getAttribute("class");
      String loadOnStartup = component.getAttribute("load-on-startup", null);
      Class clazz;
      try {
        clazz = loader.loadClass(className);
        addComponent(name, clazz, component);

        // Setup various configuration options on the scripting
        // language Cocoon component
        Interpreter interp = (Interpreter)this.select(name);
        if (interp instanceof AbstractInterpreter) {
          ((AbstractInterpreter)interp).setReloadScripts(reloadScripts);
          ((AbstractInterpreter)interp).setCheckTime(checkTime);
          if (loadOnStartup != null)
            ((AbstractInterpreter)interp).register(loadOnStartup);
        }
        if (interp instanceof Composable)
          ((Composable)interp).compose(manager);

        if (interp instanceof Loggable)
          ((Loggable)interp).setLogger(loggerUsed);
      }
      catch (Exception ex) {
        throw new ConfigurationException("Cannot load class " + className);
      }

      // If there's no default language specified through the
      // `default' attribute, set up as default the first language in
      // the sequence.
      if (defaultLanguage == null && i == 0)
        defaultLanguage = name;
    }
  }

  public void compose(ComponentManager manager)
  {
    this.manager = manager;
  }

  public String getDefaultLanguage()
  {
    return defaultLanguage;
  }
}
