package org.apache.cocoon.components.flow;

import org.apache.cocoon.environment.Environment;
import org.apache.cocoon.environment.ModifiableSource;
import org.apache.cocoon.environment.Source;

/**
 * Representation of a source in a scripting language. Loosely modeled
 * after Source and ModifiableSource, but doesn't carry all the XML
 * baggage which is not needed here.
 *
 * @author <a href="mailto:ovidiu@cup.hp.com">Ovidiu Predescu</a>
 * @since March 16, 2002
 */
public class ScriptSource
{
  Interpreter interp;
  String sourceName;
  Source source;
  boolean isModifiable;

  public ScriptSource(Interpreter interp, String sourceName)
  {
    this.interp = interp;
    this.sourceName = sourceName;
  }

  public String getSourceName()
  {
    return sourceName;
  }

  public long getLastModified()
  {
    if (isModifiable)
      ((ModifiableSource)source).refresh();

    return source == null ? 0 : source.getLastModified();
  }

  public void refresh(Environment environment)
    throws Exception
  {
    source = interp.readScript(environment, sourceName);
    isModifiable = source instanceof ModifiableSource;
  }
}
