package org.apache.cocoon.components.treeprocessor.sitemap;

import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.cocoon.components.treeprocessor.CategoryNode;
import org.apache.cocoon.components.treeprocessor.CategoryNodeBuilder;
import org.apache.cocoon.components.treeprocessor.LinkedProcessingNodeBuilder;
import org.apache.cocoon.components.treeprocessor.ProcessingNode;

public class CallNodeBuilder2 extends CallNodeBuilder
  implements LinkedProcessingNodeBuilder
{
  protected String functionName;

  public ProcessingNode buildNode(Configuration config)
    throws Exception
  {
    this.resourceName = config.getAttribute("resource", null);
    this.functionName = config.getAttribute("function", null);

    if (resourceName == null && functionName == null)
      throw new ConfigurationException("<map:call> must have either a 'resource' or a 'function' attribute!");

    if (resourceName != null)
      this.node = new CallNode();
    else
      this.node = new CallFunctionNode(functionName);

    this.treeBuilder.setupNode(this.node, config);
    if (node instanceof Configurable)
      ((Configurable)this.node).configure(config);

    return this.node;
  }

  public void linkNode()
    throws Exception
  {
    if (resourceName != null)
      super.linkNode();
    else {
      CategoryNode resources
        = CategoryNodeBuilder.getCategoryNode(this.treeBuilder, "resources");

      if (resources == null) {
        String msg = "This sitemap contains no resources. Cannot call at "
          + this.node.getLocation();
        getLogger().error(msg);
        throw new ConfigurationException(msg);
      }

      ((CallFunctionNode)this.node).setResources(resources);
    }
  }
}
