/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.scheme.servlet;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import sisc.ContinuationException;
import sisc.Interpreter;
import sisc.data.Value;

/**
 * Interactive Scheme evaluator. Use it only during development as it
 * is a big security risk.
 *
 * Receive the Scheme forms to evaluate via a POST message. Use the
 * Emacs minor mode to automatically submit a buffer or a region to
 * evaluate in the servlet.
 *
 * @author <a href="mailto:ovidiu@cup.hp.com">Ovidiu Predescu</a>
 * @since December  8, 2001
 */
public class REPLEvalServlet extends REPLGenericServlet
{
  protected boolean isEnabled = false;
  
  public void init(ServletConfig config)
    throws ServletException
  {
    super.init(config);

    String enabledString = config.getInitParameter("enabled");
    if (enabledString != null)
      isEnabled = enabledString.equalsIgnoreCase("true");
  }

  public void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    if (!isEnabled) {
      response.sendError(HttpServletResponse.SC_FORBIDDEN,
                         "You're not authorized to access this URL!");
      return;
    }

    response.setContentType("text/plain");

    OutputStream os = response.getOutputStream();
    PrintStream out = new PrintStream(os);
    System.setErr(out);

    try {
      InputStream is = request.getInputStream();
      InputStreamReader isr = new InputStreamReader(is);
      int size = 4096;
      int len;
      char[] buffer = new char[size];
      StringBuffer sbuf = new StringBuffer();

      while ((len = isr.read(buffer, 0, size)) > 0) {
        sbuf.append(buffer, 0, len);
      }

      System.out.println("executing '" + sbuf + "'");

      Interpreter interp = getInterpreter();
      out.println(interp.eval(sbuf.toString()));
      releaseInterpreter(interp);
    }
    catch (Exception ex) {
      out.println("ERROR: " + ex);
      throw new ServletException(ex);
    }
  }
}
