/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.scheme.sitemap;

import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.components.pipeline.StreamPipeline;
import org.apache.cocoon.environment.Environment;
import org.apache.cocoon.environment.Source;
import sisc.ModuleAdapter;
import sisc.data.Pair;
import sisc.data.Value;
import sisc.modules.J2S;

public abstract class CocoonModule extends ModuleAdapter
{
  static protected Parameters emptyParam = new Parameters();

  /**
   * Type cast function from a Scheme wrapper of a SchemeSitemap.
   *
   * @param scm a Scheme wrapper instance of a SchemeSitemap.
   * @return a <code>SchemeSitemap</code> value
   */
  static public SchemeSitemap schemeSitemap(Value scm)
  {
    try {
      return (SchemeSitemap)(((J2S.JavaObject)scm).o);
    }
    catch (ClassCastException ex) {
      typeError("SchemeSitemap", scm);
    }
    return null;
  }

  /**
   * Type cast function from a Scheme wrapper of an Environment
   * instance.
   *
   * @param scm a Scheme wrapper of an Environment instance.
   * @return an <code>Environment</code> value
   */
  static public Environment environment(Value scm)
  {
    try {
      return (Environment)(((J2S.JavaObject)scm).o);
    }
    catch (ClassCastException ex) {
      typeError("Environment", scm);
    }
    return null;
  }

  /**
   * Type cast function from a Scheme wrapper of an StreamPipeline
   * instance.
   *
   * @param scm a Scheme wrapper of an StreamPipeline instance.
   * @return an <code>StreamPipeline</code> value
   */
  static public StreamPipeline streamPipeline(Value scm)
  {
    try {
      return (StreamPipeline)(((J2S.JavaObject)scm).o);
    }
    catch (ClassCastException ex) {
      typeError("StreamPipeline", scm);
    }
    return null;
  }

  /**
   * Type cast function from a Scheme wrapper of
   * <code>ComponentManager</code> instance.
   *
   * @param scm a Scheme wrapper of a Cocoon
   * <code>ComponentManager</code> instance.
   * @return an <code>ComponentManager</code> value
   */
  static public ComponentManager componentManager(Value scm)
  {
    try {
      return (ComponentManager)(((J2S.JavaObject)scm).o);
    }
    catch (ClassCastException ex) {
      typeError("ComponentManager", scm);
    }
    return null;
  }

  /**
   * Type cast function from a Scheme wrapper of a Cocoon
   * <code>Source</code> instance.
   *
   * @param scm a Scheme wrapper of a Cocoon <code>Source</code>
   * instance.
   * @return an <code>Source</code> value
   */
  static public Source source(Value scm)
  {
    try {
      return (Source)(((J2S.JavaObject)scm).o);
    }
    catch (ClassCastException ex) {
      typeError("Source", scm);
    }
    return null;
  }

  /**
   * Retrieve an entry from an association list. Uses eq? to compare
   * the CAR of each entry.
   *
   * @param l the association list
   * @param v the value to be searched for
   * @return a <code>Pair</code> value representing the entry, or
   * <tt>FALSE</tt> if no such entry is present.
   */
  static public Value assq (Value l, Value v)
  {
    Pair list = pair(l);
    while (list != EMPTYLIST) {
      Pair entry = pair(list.car);
      if (entry.car.eq(v))
        return entry;
      list = pair(list.cdr);
    }
    return FALSE;
  }

  /**
   * Assumes the <tt>sparams</tt> is either an association list or the
   * FALSE value. It returns either an empty Avalon
   * <tt>Parameters</tt> instance, or a <tt>Parameters</tt> instance
   * that contains the values extracted from the association list.
   *
   * @param sparams a <code>Value</code> value, either <tt>FALSE</tt>
   * or an association list
   * @return an Avalon <code>Parameters</code> instance
   */
  static public Parameters getParameters(Value sparams)
  {
    Parameters params = emptyParam;

    if (!sparams.eq(FALSE)) {
      params = new Parameters();
      Pair sparamValues = pair(pair(sparams).cdr);
      while (sparamValues != EMPTYLIST) {
        Pair entry = pair(sparamValues.car);
        String name = string(entry.car);
        String value = string(entry.cdr);
        params.setParameter(name, value);
        sparamValues = pair(sparamValues.cdr);
      }
    }

    return params;
  }
}
