/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.scheme.sitemap;

import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.component.Composable;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.cocoon.components.pipeline.EventPipeline;
import org.apache.cocoon.components.pipeline.StreamPipeline;
import org.apache.cocoon.environment.Environment;
import org.apache.cocoon.sitemap.AbstractSitemap;
import org.apache.cocoon.sitemap.Sitemap;

/**
 * This class is responsible with configuring the ComponentManager
 * appropriately with the sitemap components to be used in
 * sitemaps. These components are read from the components.xconf file
 * at configuration time by this class.
 *
 * @author <a href="mailto:ovidiu@cup.hp.com">Ovidiu Predescu</a>
 * @since January 4, 2002
 */
public class SitemapComponents extends AbstractSitemap implements Configurable
{
  protected String defaultGenerator;
  protected String defaultTransformer;
  protected String defaultReader;
  protected String defaultSerializer;
  protected String defaultSelector;
  
  public void configure(Configuration confs)
    throws ConfigurationException
  {
    System.out.print("Loading the sitemap components... ");
    System.out.flush();

    try {
      Configuration generatorsConf = confs.getChild("generators");
      defaultGenerator = generatorsConf.getAttribute("default");
      Configuration[] gConf = generatorsConf.getChildren("generator");
      for (int i = 0; i < gConf.length; i++)        
        load_component(Sitemap.GENERATOR,
                       gConf[i].getAttribute("name", ""),
                       gConf[i].getAttribute("src"),
                       gConf[i],
                       gConf[i].getAttribute("mime-type", null));

      Configuration readersConf = confs.getChild("readers");
      defaultReader = readersConf.getAttribute("default");
      Configuration[] rConf = readersConf.getChildren("reader");
      for (int i = 0; i < rConf.length; i++)
        load_component(Sitemap.READER,
                       rConf[i].getAttribute("name", ""),
                       rConf[i].getAttribute("src"),
                       rConf[i],
                       rConf[i].getAttribute("mime-type", null));

      Configuration transformersConf = confs.getChild("transformers");
      defaultTransformer = transformersConf.getAttribute("default");
      Configuration[] tConf = transformersConf.getChildren("transformer");
      for (int i = 0; i < tConf.length; i++)
        load_component(Sitemap.TRANSFORMER,
                       tConf[i].getAttribute("name", ""),
                       tConf[i].getAttribute("src"),
                       tConf[i],
                       tConf[i].getAttribute("mime-type", null));

      Configuration serializersConf = confs.getChild("serializers");
      defaultSerializer = serializersConf.getAttribute("default");
      Configuration[] sConf = serializersConf.getChildren("serializer");
      for (int i = 0; i < sConf.length; i++)
        load_component(Sitemap.SERIALIZER,
                       sConf[i].getAttribute("name", ""),
                       sConf[i].getAttribute("src"),
                       sConf[i],
                       sConf[i].getAttribute("mime-type", null));

      Configuration selectorsConf = confs.getChild("selectors");
      defaultSelector = selectorsConf.getAttribute("default");
      Configuration[] slConf = selectorsConf.getChildren("selector");
      for (int i = 0; i < slConf.length; i++)
        load_component(Sitemap.SELECTOR,
                       slConf[i].getAttribute("name", ""),
                       slConf[i].getAttribute("src"),
                       slConf[i],
                       slConf[i].getAttribute("mime-type", null));
    }
    catch (Exception ex) {
      throw new ConfigurationException(ex.toString());
    }

    System.out.println("done");
  }

  public StreamPipeline getStreamPipeline()
    throws Exception
  {
    EventPipeline eventPipeline
      = (EventPipeline)manager.lookup(EventPipeline.ROLE);
    eventPipeline.recompose(manager);

    StreamPipeline pipeline
      = (StreamPipeline)manager.lookup(StreamPipeline.ROLE);
    pipeline.setEventPipeline(eventPipeline);
    pipeline.recompose(manager);

    return pipeline;
  }

  public void releasePipeline(StreamPipeline pipeline)
  {
    EventPipeline eventPipeline = pipeline.getEventPipeline();
    manager.release((Component)eventPipeline);
    manager.release((Component)pipeline);
  }

  public String getDefaultGeneratorType()
  {
    return defaultGenerator;
  }

  public String getDefaultTransformerType()
  {
    return defaultTransformer;
  }

  public String getDefaultReaderType()
  {
    return defaultReader;
  }

  public String getDefaultSerializerType()
  {
    return defaultSerializer;
  }

  public String getDefaultSelectorType()
  {
    return defaultSelector;
  }

  public boolean process(Environment environment)
    throws Exception
  {
    throw new Exception("Should not be invoked");
  }

  public boolean process(Environment environment, StreamPipeline pipeline,
                         EventPipeline eventPipeline)
    throws Exception
  {
    throw new Exception("Should not be invoked");
  }
}
