/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.scheme.sitemap;

import org.apache.cocoon.xml.AbstractXMLConsumer;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * SAX content handler for translating an XML document in the SXML
 * (Scheme XML) representation.
 *
 * @author <a href="mailto:ovidiu@cup.hp.com">Ovidiu Predescu</a>
 * @since January 16, 2002
 */
public class XMLtoSXML extends AbstractXMLConsumer
{
  protected StringBuffer content = new StringBuffer(100000);
  Locator locator;

  public XMLtoSXML() {}

  public String getContent()
  {
    return content.toString();
  }

  public void startDocument()
    throws SAXException
  {
    content.append("(quote ");
  }

  public void endDocument()
    throws SAXException
  {
    content.append(")");
  }

  public void startElement(String uri, String loc, String qName, Attributes a)
    throws SAXException
  {
    content.append("(" + loc + " (@ ");
    for (int i = 0, size = a.getLength(); i < size; i++) {
      String attrName = a.getQName(i);

      if (!attrName.startsWith("xmlns:"))
        content.append("(" +  attrName + " \"" + a.getValue(i) + "\") ");
    }
    content.append("(*line* " + locator.getLineNumber() + ") ");
    content.append(") ");
  }

  public void endElement(String uri, String loc, String raw)
    throws SAXException
  {
    content.append(") ");
  }

  public void characters(char ch[], int start, int len)
    throws SAXException
  {
    content.append(escape(ch, start, len));
  }

  public void setDocumentLocator(Locator locator)
  {
    this.locator = locator;
  }

  /**
   * Escapes characters in <code>chars</code> as follows:
   *
   * <ul>
   *  <li><pre>\  -> \\</pre></li>
   *  <li><pre>"  -> \"</pre></li>
   * </ul>
   *
   * @param chars a <code>char[]</code> value
   */
  public static String escape(char[] chars, int start, int len)
  {
    if (chars == null)
      return "";
    
    // Assume about 1% of the characters need to be escaped; probably
    // a safe bet in most cases. Otherwise the buffer is increased
    // automatically by the StringBuffer.
    StringBuffer escaped = new StringBuffer((int)(chars.length * 1.01));

    for (int i = start; i < len; i++) {
      char ch = chars[i];
      switch (ch) {
      case '\\':
        escaped.append("\\\\\\\\");
        break;
      case '"':
        escaped.append("\\\\\"");
        break;
      default:
        escaped.append(ch);
        break;
      }
    }

    return escaped.toString();
  }
}
