/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.sunshine.components;

import org.apache.avalon.excalibur.pool.Recyclable;
import org.apache.avalon.excalibur.source.SourceParameters;
import org.apache.avalon.framework.component.Component;
import org.apache.avalon.framework.component.ComponentException;
import org.apache.avalon.framework.component.ComponentManager;
import org.apache.avalon.framework.component.Composable;
import org.apache.avalon.framework.component.Recomposable;
import org.apache.avalon.framework.logger.AbstractLoggable;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.ResourceNotFoundException;
import org.apache.cocoon.components.RequestLifecycleComponent;
import org.apache.cocoon.environment.ObjectModelHelper;
import org.apache.cocoon.environment.Request;
import org.apache.cocoon.environment.Response;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.cocoon.sunshine.SunShine;
import org.apache.cocoon.sunshine.connector.ResourceConnector;
import org.w3c.dom.DocumentFragment;
import org.xml.sax.SAXException;
import java.io.IOException;
import java.util.Map;

/**
 * The base class for all components
 *
 * @author <a href="mailto:cziegeler@s-und-n.de">Carsten Ziegeler</a>
 * @version CVS $Id: AbstractSunShineComponent.java,v 1.2 2002/02/22 06:57:17 cziegeler Exp $
*/
public abstract class AbstractSunShineComponent
extends AbstractLoggable
implements Component, Composable, Recomposable, Recyclable, RequestLifecycleComponent {

    private SunShine          sunShineComponent;
    private ResourceConnector resourceConnectorComponent;

    protected ComponentManager manager;

    /** The current object model */
    protected Map     objectModel;

    /** The current source resolver */
    protected SourceResolver resolver;

    protected Request          request;
    protected Response         response;


    /**
     * Composer interface. Get the Avalon ComponentManager.
     */
    public void compose(ComponentManager manager) {
        this.manager = manager;
    }

    /**
     * Recomposable
     */
    public void recompose( ComponentManager componentManager )
    throws ComponentException {
        this.recycle();
        this.manager = componentManager;
    }

    /**
     * Set the <code>SourceResolver</code>, objectModel <code>Map</code>,
     * used to process the request.
     *  Set up the sunShine component.
     *  This method is automatically called for each request. Do not invoke
     *  this method by hand.
     */
    public void setup(SourceResolver resolver, Map objectModel)
    throws ProcessingException, SAXException, IOException {
        this.objectModel = objectModel;
        this.resolver    = resolver;
        this.request = ObjectModelHelper.getRequest(objectModel);
        this.response = ObjectModelHelper.getResponse(objectModel);
    }

    /**
     * Get the sunShine component
     */
    protected SunShine getSunShineComponent()
    throws ProcessingException {
        if (this.sunShineComponent == null) {
            try {
                this.sunShineComponent = (SunShine)this.manager.lookup(SunShine.ROLE);
            } catch (ComponentException ce) {
                throw new ProcessingException("Error during lookup of sunShine component.", ce);
            }
        }
        return this.sunShineComponent;
    }

    /**
     * Get the resource connector
     */
    protected ResourceConnector getResourceConnector()
    throws ProcessingException {
        if (this.resourceConnectorComponent == null) {
            try {
                this.resourceConnectorComponent = (ResourceConnector)this.manager.lookup(ResourceConnector.ROLE);
            } catch (ComponentException ce) {
                throw new ProcessingException("Error during lookup of sunShine resource connector.", ce);
            }
        }
        return this.resourceConnectorComponent;
    }

    /**
     * Recycle
     */
    public void recycle() {
        if (this.manager != null) {
            this.manager.release(this.sunShineComponent);
            this.manager.release(this.resourceConnectorComponent);
        }
        this.sunShineComponent = null;
        this.resourceConnectorComponent = null;
        this.objectModel = null;
        this.resolver = null;
        this.request = null;
        this.response = null;
    }

    /**
     * Load a resource.
     * The resource connector usually throws a ResourceNotFound exception
     * when a pipeline is not found. This exception is catched here
     * and rethrown as a usual ProcessingException with the given message!
     */
    public DocumentFragment loadXML(int                resourceType,
                                    Parameters         typeParameters,
                                    String             resourceIdentifier,
                                    SourceParameters   resourceParameters,
                                    String             errorMessage)
    throws ProcessingException {
        try {
            return this.getResourceConnector().loadXML(resourceType, typeParameters,
                                                       resourceIdentifier, resourceParameters);
        } catch (ResourceNotFoundException rnfe) {
            throw new ProcessingException(errorMessage, rnfe);
        }
    }
}