/*

 ============================================================================
                   The Apache Software License, Version 1.1
 ============================================================================

 Copyright (C) 1999-2002 The Apache Software Foundation. All rights reserved.

 Redistribution and use in source and binary forms, with or without modifica-
 tion, are permitted provided that the following conditions are met:

 1. Redistributions of  source code must  retain the above copyright  notice,
    this list of conditions and the following disclaimer.

 2. Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

 3. The end-user documentation included with the redistribution, if any, must
    include  the following  acknowledgment:  "This product includes  software
    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
    Alternately, this  acknowledgment may  appear in the software itself,  if
    and wherever such third-party acknowledgments normally appear.

 4. The names "Apache Cocoon" and  "Apache Software Foundation" must  not  be
    used to  endorse or promote  products derived from  this software without
    prior written permission. For written permission, please contact
    apache@apache.org.

 5. Products  derived from this software may not  be called "Apache", nor may
    "Apache" appear  in their name,  without prior written permission  of the
    Apache Software Foundation.

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 This software  consists of voluntary contributions made  by many individuals
 on  behalf of the Apache Software  Foundation and was  originally created by
 Stefano Mazzocchi  <stefano@apache.org>. For more  information on the Apache
 Software Foundation, please see <http://www.apache.org/>.

*/
package org.apache.cocoon.sunshine.transformation;

import java.io.IOException;
import java.util.Map;
import java.util.Properties;

import org.apache.avalon.excalibur.source.SourceParameters;
import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.sunshine.connector.*;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import org.w3c.dom.DocumentFragment;

import javax.xml.transform.OutputKeys;

/**
 * This is the sunShine connect transformer.
 * The class is abstract as the SunShinePreTransformer inherits from
 * this class.
 *
 * @author <a href="mailto:cziegeler@s-und-n.de">Carsten Ziegeler</a>
 * @version CVS $Id: SunShineConnectTransformer.java,v 1.2 2002/02/22 06:57:19 cziegeler Exp $
*/
public abstract class SunShineConnectTransformer
extends AbstractSunShineTransformer{

    public static final String INCLUDE_XML_ELEMENT = "include";
    public static final String CONNECTION_ELEMENT	= "connection";
    public static final String CTARGET_ELEMENT		= "target";
    public static final String CCONFIGLIST_ELEMENT      = "config";
    public static final String CPARAMLIST_ELEMENT 	= "params";
    public static final String CPARAM_ELEMENT		= "param";
    public static final String CPARAMNAME_ELEMENT 	= "name";
    public static final String CPARAMVALUE_ELEMENT 	= "value";


    private SourceParameters callParams;
    private Parameters configParams;

    private static final int STATE_OUTSIDE   = 0;
    private static final int STATE_INCLUDE   = 1;
    private static final int STATE_CONNECTION= 2;
    private static final int STATE_FILE      = 3;
    private static final int STATE_RESOURCE  = 4;

    private int state;

    /**
     * Setup the transformer
     */
    public void setupTransforming()
    throws ProcessingException, SAXException, IOException {
        super.setupTransforming();
        this.state = SunShineConnectTransformer.STATE_OUTSIDE;
    }

    /**
     * Recycle the component
     */
    public void recycle() {
        super.recycle();
        this.callParams = null;
        this.configParams = null;
    }

    /**
     * This is the real implementation of the startElement event for the sunShine Transformer
     * The event is checked for a valid sunShine element and the corresponding command
     * is executed.
     */
    public void startTransformingElement(String uri,
                                         String name,
                                         String raw,
                                         Attributes attr)
    throws ProcessingException, IOException, SAXException {
        if (this.getLogger().isDebugEnabled() == true) {
            this.getLogger().debug("BEGIN startTransformingElement uri=" + uri +
                                   ", name=" + name + ", raw=" + raw + ", attr=" + attr + ")");
        }

        // Element: include
        if (name.equals(INCLUDE_XML_ELEMENT) == true
                   && this.state == STATE_OUTSIDE) {
            this.state = SunShineConnectTransformer.STATE_INCLUDE;
            String ignoreErrors = attr.getValue("ignoreErrors");
            if (ignoreErrors == null || ignoreErrors.length() == 0) {
                ignoreErrors = "false";
            }
            this.stack.push(new Boolean(this.ignoreEmptyCharacters));
            this.stack.push(new Boolean(this.ignoreWhitespaces));
            this.stack.push(ignoreErrors);
            this.ignoreEmptyCharacters = false;
            this.ignoreWhitespaces = true;

        } else if (name.equals(CONNECTION_ELEMENT) == true
                 && this.state == SunShineConnectTransformer.STATE_INCLUDE) {
            this.state = SunShineConnectTransformer.STATE_CONNECTION;

        // target
        } else if (name.equals(CTARGET_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            this.startTextRecording();

        // configparameters
        } else if (name.equals(CCONFIGLIST_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            stack.push("CONFIGPARAMEND");

        // parameters
        } else if (name.equals(CPARAMLIST_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            stack.push("PARAMEND");

        // parameter
        } else if (name.equals(CPARAM_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {

        // parameter name
        } else if (name.equals(CPARAMNAME_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            this.startTextRecording();

        // parameter value
        } else if (name.equals(CPARAMVALUE_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            Properties format = new Properties();
            format.put(OutputKeys.METHOD, "xml");
            format.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
            this.startSerializedXMLRecording(format);

        // unknown
        } else {
            super.startTransformingElement(uri, name, raw, attr);
        }

        if (this.getLogger().isDebugEnabled() == true) {
            this.getLogger().debug("END startTransformingElement");
        }
    }

    public void endTransformingElement(String uri,
                                       String name,
                                       String raw)
    throws ProcessingException ,IOException, SAXException {
        if (this.getLogger().isDebugEnabled() == true) {
            this.getLogger().debug("BEGIN endTransformingElement uri=" + uri +
                                   ", name=" + name + ", raw=" + raw);
        }

        if (name.equals(CONNECTION_ELEMENT) == true
            && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            // We should have everything on the stack now
            String target = (String)stack.pop();

            this.stack.push(target);

            this.state = SunShineConnectTransformer.STATE_INCLUDE;

        // Element: include
        } else if (name.equals(INCLUDE_XML_ELEMENT) == true
                   && this.state == STATE_INCLUDE) {
            this.state = SunShineConnectTransformer.STATE_OUTSIDE;

            final String resource = (String)stack.pop();

            final boolean ignoreErrors = ((String)stack.pop()).equals("true");

            final Resource resourceObject = new Resource(this.resolver, resource);

            try {
                this.getResourceConnector().streamXML(resourceObject.getResourceType(), configParams,
                      resourceObject.getResourceIdentifier(), callParams,
                      this, this);
            } catch (ProcessingException sse) {
                if (ignoreErrors == false) throw sse;
            }

            this.ignoreWhitespaces = ((Boolean)stack.pop()).booleanValue();
            this.ignoreEmptyCharacters = ((Boolean)stack.pop()).booleanValue();

        } else if (name.equals(CTARGET_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            stack.push(this.endTextRecording());

        } else if (name.equals(CCONFIGLIST_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            configParams = new Parameters();
            // Now get the parameters off the stack
            String label = (String)stack.pop();
            String value="";
            while (!label.equals("CONFIGPARAMEND")) {
                value = (String)stack.pop();
                configParams.setParameter((String)stack.pop(), value);
                label = (String)stack.pop();
            }

        } else if (name.equals(CPARAMLIST_ELEMENT) == true
                 && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            callParams = new SourceParameters();
            // Now get the parameters off the stack
            String label = (String)stack.pop();
            String value="";
            while (!label.equals("PARAMEND")) {
                value = (String)stack.pop();
                callParams.setParameter((String)stack.pop(), value);
                label = (String)stack.pop();
            }

        } else if (name.equals(CPARAM_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
        } else if (name.equals(CPARAMNAME_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            stack.push(this.endTextRecording());

        // parameter value
        } else if (name.equals(CPARAMVALUE_ELEMENT) == true
                   && this.state == SunShineConnectTransformer.STATE_CONNECTION) {
            stack.push(this.endSerializedXMLRecording());
            stack.push("xml");

        } else {
            super.endTransformingElement(uri, name, raw);
        }

        if (this.getLogger().isDebugEnabled() == true) {
            this.getLogger().debug("END endTransformingElement");
        }
    }
}
