<?xml version="1.0"?>

<xsl:stylesheet version="1.0"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:dir="http://apache.org/cocoon/directory/2.0"
 xmlns:source="http://apache.org/cocoon/source/1.0">


<!-- generic behaviour templates -->

	<xsl:template match="instance/*" mode="sub-editor">
		<xsl:apply-templates select="."/>	
  </xsl:template>

	<xsl:template match="instance[@behaviour='see']/page" mode="sub-editor">
		<div class="block">
			<span class="tab">Editor View</span>
			<div class="content"><xsl:apply-templates mode="editor-view"/></div>
		</div>
  </xsl:template>


<!-- editor-view templates to display your doctype -->

	<xsl:template match="content" mode="editor-view">
		<div><xsl:apply-templates mode="editor-view"/></div>
  </xsl:template>
  
	<xsl:template match="title" mode="editor-view">
		<div class="title"><xsl:apply-templates/></div>
  </xsl:template>
  
	<xsl:template match="para" mode="editor-view">
		<p><xsl:apply-templates/></p>
  </xsl:template>


<!-- Displays the directory listing -->
  <xsl:template match="dir:directory">
   <div class="block">
		<span class="tab"><a href="../" title="{$config/parent-dir/@tip}"><b><xsl:value-of select="$config/parent-dir/text()"/></b></a></span>
		<div class="content">
			<table width="100%">
				<xsl:apply-templates/>
			</table>
		</div>
   </div>
  </xsl:template>

<!-- Displays sub-directories -->
  <xsl:template match="dir:directory/dir:directory">
   <tr>
   	<xsl:if test="position() mod 2 = 0"><xsl:attribute name="id">row</xsl:attribute></xsl:if>
   	<td>
   		<a href="{@name}/" title="{$config/command[@behaviour = 'dir']/@tip}">
      	<img src="{concat('image/',$config/command[@behaviour = 'dir']/@icon)}" border="0"/>
    	</a>
    	<a href="{@name}/" title="{$config/command[@behaviour = 'dir']/@tip}"><xsl:value-of select="@name"/></a>
    </td>
    <td>Directory</td>
    <td><xsl:value-of select="@date"/></td>
	 </tr>
  </xsl:template>

<!-- Displays files in the directory listing -->
  <xsl:template match="dir:file">
   <tr>
   	<xsl:if test="position() mod 2 = 0"><xsl:attribute name="id">row</xsl:attribute></xsl:if>
    <td>
     <a title="{$config/command[@behaviour = 'see']/@tip}">
      <xsl:attribute name="href"><xsl:value-of select="$command-base"/><xsl:value-of select="$config/command[@behaviour = 'see']/text()"/><xsl:value-of select="$target"/><xsl:value-of select="@name"/></xsl:attribute>
      <img src="{concat('image/',$config/command[@behaviour = 'see']/@icon)}" border="0"/>
     </a>
     <a title="{$config/command[@behaviour = 'see']/@tip}">
      <xsl:attribute name="href"><xsl:value-of select="$command-base"/><xsl:value-of select="$config/command[@behaviour = 'see']/text()"/><xsl:value-of select="$target"/><xsl:value-of select="@name"/></xsl:attribute>
      <xsl:value-of select="@name"/>
     </a>
    </td>
    <td><xsl:value-of select="dir:xpath/title"/></td>
    <td><xsl:value-of select="@date"/></td>
   </tr>
  </xsl:template>

	<xsl:template match="validationResult">
		<div class="block">
			<span class="tab">Schematron Validation</span>
			<div class="content">
				<table>
					<xsl:for-each select="pattern">
						<tr>
							<td class="col1"><xsl:value-of select="@name"/></td>
							<td class="col2">
								<xsl:choose>
									<xsl:when test="error">
										<span class="failure">failure</span>
									</xsl:when>
									<xsl:otherwise><span class="success">success</span></xsl:otherwise>
								</xsl:choose>
							</td>
						</tr>
						<xsl:if test="error">
							<xsl:for-each select="error">
								<tr class="error">
									<td/>
									<td class="col1"> <xsl:value-of select="path"/></td>
									<td class="col2"><xsl:value-of select="message"/></td>
								</tr>				
							</xsl:for-each>
						</xsl:if>
					</xsl:for-each>
				</table>
			</div>
		</div>
  </xsl:template>
 
<!-- outputs the response from WriteableSourceTransformer -->
  <xsl:template match="source:write">
		<div class="block">
			<span class="tab">Writing to Source</span>
			<div class="content">
				<table>
					<tr>
						<td class="col1">Result</td>
						<td class="col2"><span class="{@result}"><xsl:value-of select="@result"/></span></td>
					</tr>
					<xsl:if test="@result = 'failed'">
						<tr class="error">
							<td class="col1">Error</td>
							<td class="col2"><span class="{@result}"><xsl:value-of select="text()"/></span></td>
						</tr>					
					</xsl:if>
					<tr>
						<td class="col1">Source</td>
						<td class="col2"><xsl:value-of select="@src"/></td>
					</tr>
					<tr>
						<td class="col1">Action</td>
						<td class="col2">
							<xsl:choose>
								<xsl:when test="@action = 'none'">None taken</xsl:when>
								<xsl:when test="@action = 'new'">New file created</xsl:when>
								<xsl:when test="@action = 'overwritten'">Existing file overwritten</xsl:when>
								<xsl:otherwise>Unknown</xsl:otherwise>
							</xsl:choose>
						</td>
					</tr>
					<xsl:if test="@serializer">
						<tr>
							<td class="col1">Serializer</td>
							<td class="col2"><xsl:value-of select="@serializer"/></td>
						</tr>
					</xsl:if>
				</table>
			</div>
		</div>
  </xsl:template>

</xsl:stylesheet>
