<!-- sch:key moved as a  child  of sch:schema and syntax changed to xslt usage -->
<!-- 
  This stylesheet is derived from the original work of Miloslav Nic [ nicmila@systinet.com ] 
  http://www.zvon.org/xxl/SchematronTutorial/General/contents.html

  It is offered in its current form by Ivelin Ivanov [ ivelin@iname.com ]
-->

<xsl:stylesheet version="1.0"
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
      xmlns:sch="http://www.ascc.net/xml/schematron"
      xmlns:aaa="http://www.w3.org/1999/XSL/TransformAlias"
      xmlns:zvon="http://zvon.org/schematron">

<xsl:import href="schematron-wrappers.xsl"/>
<xsl:import href="schematron-extensions.xsl"/>



<xsl:namespace-alias stylesheet-prefix="aaa" result-prefix="xsl"/>
<xsl:output method="xml" indent="yes"/>

<xsl:param name="outputMethod">xml</xsl:param>

<!-- root of an XML Form document -->
<xsl:variable name="docRoot">/editor/instance/*</xsl:variable>

<xsl:key name="PHASE" match="sch:phase" use="sch:active/@pattern"/>

<xsl:template name="sch-pattern">
    <xsl:apply-templates select="/sch:schema/sch:phase[@id=$phase]/sch:active[@pattern = current()/@id]"/>
    <xsl:apply-templates select="sch:p"/>
   
    <aaa:apply-templates select="." mode="{generate-id(.)}"/>
</xsl:template>

<xsl:template match="sch:pattern">
  <xsl:call-template name="patternWrapper"/>
</xsl:template>

<xsl:template match="sch:rule[not(@abstract='yes')]">
  <xsl:variable name="md" select="generate-id(parent::sch:pattern)"/>

  <aaa:template match="{@context}" mode="{$md}" priority="10">
    <xsl:apply-templates select="*|//sch:rule[@abstract='yes' and @id = current()/sch:extends/@rule]/*"/>
    <aaa:apply-templates select="*|@*" mode="{$md}"/>
  </aaa:template>
</xsl:template>

<xsl:template match="sch:key" mode="defineKey">
  <aaa:key name="{@name}" match="{@match}" use="{@use}"/>
  <!--   <aaa:key name="{@name}" match="{parent::sch:rule/@context}" use="{@path}"/> -->
</xsl:template>

<xsl:template match="sch:report | sch:assert">
  <xsl:variable name="startNOT">
    <xsl:if test="self::sch:assert">not(</xsl:if>
  </xsl:variable>
  <xsl:variable name="endNOT">
    <xsl:if test="self::sch:assert">)</xsl:if>
  </xsl:variable>

  <aaa:if test="{$startNOT}{@test}{$endNOT}">   
    <xsl:call-template name="reportAssertWrapper"/>
  </aaa:if>
</xsl:template>

<xsl:template match="@*" mode="addAttributes">
  <xsl:copy/>
</xsl:template>

<xsl:template match="sch:name">
  <xsl:choose>
    <xsl:when test="@zvon:fullPath='yes'">
      <aaa:apply-templates select="." mode="fullPath"/>
    </xsl:when>
    <xsl:otherwise><aaa:value-of select="name({@path})"/>
</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="sch:schema">
  <aaa:template match="{$docRoot}">
    <xsl:call-template name="documentWrapper"/> 
    <xsl:call-template name="schemaDiagnosticWrapper"/>
  </aaa:template>
  <xsl:apply-templates select="sch:pattern/sch:rule"/>
</xsl:template>


<xsl:template match="sch:schema" mode="process">
  <xsl:apply-templates select="sch:p | sch:pattern[$phase='#ALL'] | sch:pattern[key('PHASE',@id)/@id = $phase]"/>
</xsl:template>


<xsl:template match="sch:schema" mode="diagnostic">
  <xsl:apply-templates select="sch:diagnostics/sch:diagnostic"/>
  <xsl:apply-templates select="sch:pattern" mode="diagnostics"/>
</xsl:template>

<xsl:template match="sch:pattern" mode="diagnostics">
  
  <xsl:for-each select="sch:rule[sch:report/@diagnostics|sch:assert/@diagnostics]">
    <aaa:template match="{@context}" mode="dia_{generate-id(parent::sch:pattern)}" priority="10">
      <xsl:for-each select="sch:report[@diagnostics] | sch:assert[@diagnostics]">
	<xsl:variable name="startNOT">
	  <xsl:if test="self::sch:assert">not(</xsl:if>
	</xsl:variable>
	<xsl:variable name="endNOT">
	  <xsl:if test="self::sch:assert">)</xsl:if>
	</xsl:variable>
	<aaa:if test="{$startNOT}{@test}{$endNOT}">   
	  <xsl:call-template name='outputDiagnosticAssertReportWrapper'/>
	</aaa:if>
      </xsl:for-each>
    </aaa:template>
  </xsl:for-each>
</xsl:template>



<xsl:template name="diagnosticItems">
  <xsl:param name="value"/>
  <xsl:variable name="start" select="substring-before($value,' ')"/>
  
  <xsl:choose>
    <xsl:when test="$start=''">
      <aaa:call-template name="diagnostic_{$value}"/>
    </xsl:when>
    <xsl:otherwise>
      <aaa:call-template name="diagnostic_{$start}"/>
      <xsl:call-template name="diagnosticItems">
	<xsl:with-param name="value" select="substring-after($value,' ')"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>

</xsl:template>

<xsl:template match="sch:diagnostic">
  <aaa:template name="diagnostic_{@id}">
    <xsl:call-template name="diagnosticWrapper"/>
  </aaa:template>
</xsl:template>

<xsl:template match="sch:value-of">
  <aaa:value-of select="{@select}"/>
</xsl:template>

<xsl:template match="*"/>

<xsl:template name='applyRules'>
  <aaa:apply-templates select="{$docRoot}" mode="{generate-id(.)}"/>
</xsl:template>

<xsl:template match="/">
  <aaa:stylesheet version="1.0" >

    <xsl:for-each select="/sch:schema/namespace::*">
      <xsl:variable name="prefix" select="name()"/>
      <xsl:if test="not($prefix='xml' or $prefix='')">
	<xsl:attribute name="{$prefix}:{$prefix}" namespace="{.}">
	  <xsl:value-of select="."/>
	</xsl:attribute>
      </xsl:if>
    </xsl:for-each>

    <xsl:for-each select="/sch:schema/sch:ns">
      <xsl:if test="not(@prefix='xml' or @prefix='')">
	<xsl:attribute name="{@prefix}:{@prefix}" namespace="{@uri}">
	  <xsl:value-of select="@uri"/>
	</xsl:attribute>
      </xsl:if>
    </xsl:for-each>

    <aaa:output method="{$outputMethod}" indent="yes"/>


      <xsl:apply-templates select="/sch:schema/sch:key" mode="defineKey"/>
      <xsl:apply-templates select="sch:schema"/>

      <xsl:for-each select="/sch:schema/sch:pattern">
	<aaa:template match="text()" mode="{generate-id()}"/>

	<aaa:template match="*|@*" mode="{generate-id()}">
	  <aaa:apply-templates select="node()|@*" mode="{generate-id()}"/>
	</aaa:template>

	<aaa:template match="text()" mode="dia_{generate-id()}"/>

	<aaa:template match="*|@*" mode="dia_{generate-id()}">
	  <aaa:apply-templates select="node()|@*" mode="dia_{generate-id()}"/>
	</aaa:template>
      </xsl:for-each>

      <aaa:template match="{$docRoot}" mode="startDiagnostics">
	<xsl:for-each select="/sch:schema/sch:pattern">
	  <aaa:apply-templates select="{$docRoot}" mode="dia_{generate-id(.)}"/>
	</xsl:for-each>
      </aaa:template>

      <xsl:apply-templates select="sch:schema" mode="diagnostic"/>

      <xsl:call-template name="fullPath"/>
    </aaa:stylesheet>
</xsl:template>

<xsl:template match="sch:dir">
  <xsl:call-template name="dirWrapper"/>
</xsl:template>

<xsl:template match="sch:span">
  <xsl:call-template name="spanWrapper"/>
</xsl:template>


<xsl:template match="sch:p">
  <xsl:call-template name="pWrapper"/>
</xsl:template>

<xsl:template match="@icon">
  <xsl:call-template name="iconWrapper"/>
</xsl:template>

<xsl:template match="sch:emph">
  <xsl:call-template name="emphWrapper"/>
</xsl:template>

<xsl:template match="sch:title">
  <xsl:apply-templates/>
</xsl:template>

<xsl:param name="phase">
  <xsl:choose>
    <xsl:when test="/sch:schema/@defaultPhase">
      <xsl:value-of select="/sch:schema/@defaultPhase"/>
    </xsl:when>
    <xsl:otherwise>#ALL</xsl:otherwise>
  </xsl:choose>
</xsl:param>

<xsl:template match="sch:active">
  <xsl:apply-templates select="text() | sch:dir | sch:span | sch:emph"/>
</xsl:template>


</xsl:stylesheet>















