<xsl:stylesheet version="1.0"
      xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
      xmlns:sch="http://www.ascc.net/xml/schematron"
      xmlns:aaa="http://www.w3.org/1999/XSL/TransformAlias"
      xmlns:zvon="http://zvon.org/schematron"
>

<!--
	Schematron wrappers for the slash-edit
	
	originally copied from work by:
	Ivelin Ivanov, ivelin@iname.com
-->

<!-- root of an XML Form document -->
<xsl:variable name="docRootPrefix">/editor/instance/</xsl:variable>

<xsl:template name="documentWrapper">
  <editor>
  	<aaa:copy-of select="../../@*"/> <!-- copy 'editor' attributes-->
    <instance>
    	<aaa:copy-of select="../@*"/> <!-- copy 'instance' attributes-->
      <aaa:copy-of select="."/>
    </instance>
    <validationResult>
      <xsl:apply-templates select="." mode="process"/>
    </validationResult>
  </editor>
</xsl:template>

<xsl:template name="schemaTitleWrapper">
  <xsl:if test="sch:title">
    <h2>
      <xsl:apply-templates select="sch:title"/>
    </h2>
  </xsl:if>
</xsl:template>


<xsl:template name="patternWrapper">
  <pattern name="{@name}">
    <xsl:call-template name="applyRules"/>
  </pattern>
</xsl:template>


<xsl:template name="reportAssertWrapper">
  <error>
    <path>
      <aaa:variable name="fpath">
        <aaa:apply-templates select="." mode="fullPath"/>
      </aaa:variable>
      <aaa:value-of select="substring-after($fpath, '{$docRootPrefix}')"/>
    </path>
    <message>
      <xsl:apply-templates/>
    </message>
  </error>
</xsl:template>

<xsl:template name="patternTitleWrapper">
  <tr>
    <th class="pattern">
      <a name="{@id}">&#160;</a>
      <xsl:text>Pattern: </xsl:text>
      <span class="patternTitle"><xsl:value-of select="@name"/></span>
    </th>
  </tr>
</xsl:template>

<xsl:template name="patternSeeWrapper">
  <xsl:if test="@see">
    <tr>
      <td>
	<xsl:text>Documentation: </xsl:text>
	<a href="{@see}"> see here </a>
      </td>
    </tr>
  </xsl:if>
</xsl:template>

<xsl:template name="pWrapper">
  <div>

    <xsl:copy-of select="@class|@id"/>
    <xsl:apply-templates select="@icon"/>
    <xsl:apply-templates/>
  </div>
</xsl:template>

<xsl:template name="iconWrapper">
  <img src="{.}"/>
</xsl:template>

<xsl:template name="emphWrapper">
  <em><xsl:apply-templates/></em>
</xsl:template>

<xsl:template name="spanWrapper">
  <span>
    <xsl:copy-of select="@class"/>
    <xsl:apply-templates/>
  </span>
</xsl:template>

<xsl:template name="dirWrapper">
  <span dir="{@value}">
    <xsl:apply-templates/>
  </span>
</xsl:template>

<xsl:template name="zvonDiagnostics-URI">

</xsl:template>

<xsl:template name="schemaDiagnosticWrapper">
  <xsl:choose>
    <xsl:when test="parent::*/@zvon:diagnostics-URI">
      <!--<saxon:output file="{parent::*/@zvon:diagnostics-URI}">-->
	<aaa:apply-templates select="/" mode="startDiagnostics"/>
      <!--</saxon:output>-->
    </xsl:when>
    <xsl:otherwise>
      <aaa:message>
	<aaa:apply-templates select="/" mode="startDiagnostics"/>
      </aaa:message>
    </xsl:otherwise>
  </xsl:choose>
  

</xsl:template>

<xsl:template name="outputDiagnosticAssertReportWrapper">
  <xsl:choose>
    <xsl:when test="@zvon:diagnostics-URI">
      <!--<saxon:output file="{@zvon:diagnostics-URI}">-->
	<xsl:call-template name="diagnosticItems">
	  <xsl:with-param name="value" select="normalize-space(@diagnostics)"/>
	</xsl:call-template>
      <!--</saxon:output>-->
    </xsl:when>
    <xsl:otherwise>
	<xsl:call-template name="diagnosticItems">
	  <xsl:with-param name="value" select="normalize-space(@diagnostics)"/>
	</xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template> 

<xsl:template name="diagnosticWrapper">
  <xsl:text> 
Diagnostics: 
</xsl:text>
  <xsl:apply-templates/>
</xsl:template>

</xsl:stylesheet>













